"""
Tests for the Performable template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context

from analytical.templatetags.performable import PerformableNode
from analytical.tests.utils import TagTestCase, override_settings, SETTING_DELETED
from analytical.utils import AnalyticalException


@override_settings(PERFORMABLE_API_KEY='123ABC')
class PerformableTagTestCase(TagTestCase):
    """
    Tests for the ``performable`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('performable', 'performable')
        self.assertTrue('/performable/pax/123ABC.js' in r, r)

    def test_node(self):
        r = PerformableNode().render(Context())
        self.assertTrue('/performable/pax/123ABC.js' in r, r)

    @override_settings(PERFORMABLE_API_KEY=SETTING_DELETED)
    def test_no_api_key(self):
        self.assertRaises(AnalyticalException, PerformableNode)

    @override_settings(PERFORMABLE_API_KEY='123 ABC')
    def test_wrong_account_number(self):
        self.assertRaises(AnalyticalException, PerformableNode)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = PerformableNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Performable disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify(self):
        r = PerformableNode().render(Context({'user': User(username='test')}))
        self.assertTrue('_paq.push(["identify", {identity: "test"}]);' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = PerformableNode().render(Context({'user': AnonymousUser()}))
        self.assertFalse('_paq.push(["identify", ' in r, r)


class PerformableEmbedTagTestCase(TagTestCase):
    """
    Tests for the ``performable_embed`` template tag.
    """

    def test_tag(self):
        domain = 'example.com'
        page = 'test'
        r = self.render_tag('performable', 'performable_embed "%s" "%s"'
                % (domain, page))
        self.assertTrue(
                "$f.initialize({'host': 'example.com', 'page': 'test'});" in r,
                r)



"""
Tests for the Woopra template tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context

from analytical.templatetags.woopra import WoopraNode
from analytical.tests.utils import TagTestCase, override_settings, \
        SETTING_DELETED
from analytical.utils import AnalyticalException


@override_settings(WOOPRA_DOMAIN='example.com')
class WoopraTagTestCase(TagTestCase):
    """
    Tests for the ``woopra`` template tag.
    """

    def test_tag(self):
        r = self.render_tag('woopra', 'woopra')
        self.assertTrue('var woo_settings = {"domain": "example.com"};' in r, r)

    def test_node(self):
        r = WoopraNode().render(Context({}))
        self.assertTrue('var woo_settings = {"domain": "example.com"};' in r, r)

    @override_settings(WOOPRA_DOMAIN=SETTING_DELETED)
    def test_no_domain(self):
        self.assertRaises(AnalyticalException, WoopraNode)

    @override_settings(WOOPRA_DOMAIN='this is not a domain')
    def test_wrong_domain(self):
        self.assertRaises(AnalyticalException, WoopraNode)

    @override_settings(WOOPRA_IDLE_TIMEOUT=1234)
    def test_idle_timeout(self):
        r = WoopraNode().render(Context({}))
        self.assertTrue('var woo_settings = {"domain": "example.com", '
                '"idle_timeout": "1234"};' in r, r)

    def test_custom(self):
        r = WoopraNode().render(Context({'woopra_var1': 'val1',
                'woopra_var2': 'val2'}))
        self.assertTrue('var woo_visitor = {"var1": "val1", "var2": "val2"};'
                in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_name_and_email(self):
        r = WoopraNode().render(Context({'user': User(username='test',
                first_name='Firstname', last_name='Lastname',
                email="test@example.com")}))
        self.assertTrue('var woo_visitor = {"name": "Firstname Lastname", '
                '"email": "test@example.com"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_username_no_email(self):
        r = WoopraNode().render(Context({'user': User(username='test')}))
        self.assertTrue('var woo_visitor = {"name": "test"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_no_identify_when_explicit_name(self):
        r = WoopraNode().render(Context({'woopra_name': 'explicit',
                'user': User(username='implicit')}))
        self.assertTrue('var woo_visitor = {"name": "explicit"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_no_identify_when_explicit_email(self):
        r = WoopraNode().render(Context({'woopra_email': 'explicit',
                'user': User(username='implicit')}))
        self.assertTrue('var woo_visitor = {"email": "explicit"};' in r, r)

    @override_settings(ANALYTICAL_AUTO_IDENTIFY=True)
    def test_identify_anonymous_user(self):
        r = WoopraNode().render(Context({'user': AnonymousUser()}))
        self.assertTrue('var woo_visitor = {};' in r, r)

    @override_settings(ANALYTICAL_INTERNAL_IPS=['1.1.1.1'])
    def test_render_internal_ip(self):
        req = HttpRequest()
        req.META['REMOTE_ADDR'] = '1.1.1.1'
        context = Context({'request': req})
        r = WoopraNode().render(context)
        self.assertTrue(r.startswith(
                '<!-- Woopra disabled on internal IP address'), r)
        self.assertTrue(r.endswith('-->'), r)



"""
Tests for the UserVoice tags and filters.
"""

from django.contrib.auth.models import User, AnonymousUser
from django.http import HttpRequest
from django.template import Context

from analytical.templatetags.uservoice import UserVoiceNode
from analytical.tests.utils import TagTestCase, override_settings, \
        SETTING_DELETED
from analytical.utils import AnalyticalException


@override_settings(USERVOICE_WIDGET_KEY='abcdefghijklmnopqrst')
class UserVoiceTagTestCase(TagTestCase):
    """
    Tests for the ``uservoice`` template tag.
    """

    def test_node(self):
        r = UserVoiceNode().render(Context())
        self.assertTrue("'widget.uservoice.com/abcdefghijklmnopqrst.js'" in r,
                r)

    def test_tag(self):
        r = self.render_tag('uservoice', 'uservoice')
        self.assertTrue("'widget.uservoice.com/abcdefghijklmnopqrst.js'" in r,
                r)

    @override_settings(USERVOICE_WIDGET_KEY=SETTING_DELETED)
    def test_no_key(self):
        self.assertRaises(AnalyticalException, UserVoiceNode)

    @override_settings(USERVOICE_WIDGET_KEY='abcdefgh ijklmnopqrst')
    def test_invalid_key(self):
        self.assertRaises(AnalyticalException, UserVoiceNode)

    @override_settings(USERVOICE_WIDGET_KEY='')
    def test_empty_key(self):
        r = UserVoiceNode().render(Context())
        self.assertFalse("widget.uservoice.com" in r, r)

    @override_settings(USERVOICE_WIDGET_KEY='')
    def test_overridden_empty_key(self):
        vars = {'uservoice_widget_key': 'bcdefghijklmnopqrstu'}
        r = UserVoiceNode().render(Context(vars))
        self.assertTrue("'widget.uservoice.com/bcdefghijklmnopqrstu.js'" in r,
                r)

    def test_overridden_key(self):
        vars = {'uservoice_widget_key': 'defghijklmnopqrstuvw'}
        r = UserVoiceNode().render(Context(vars))
        self.assertTrue("'widget.uservoice.com/defghijklmnopqrstuvw.js'" in r,
                r)

    def test_link(self):
        r = self.render_tag('uservoice', 'uservoice_popup')
        self.assertEqual(r, "UserVoice.showPopupWidget();")

    def test_link_with_key(self):
        r = self.render_tag('uservoice',
                'uservoice_popup "efghijklmnopqrstuvwx"')
        self.assertEqual(r, 'UserVoice.showPopupWidget({"widget_key": '
                '"efghijklmnopqrstuvwx"});')

    def test_link_disables_tab(self):
        r = self.render_template(
                '{% load uservoice %}{% uservoice_popup %}{% uservoice %}')
        self.assertTrue("UserVoice.showPopupWidget();" in r, r)
        self.assertTrue('"enabled": false' in r, r)
        self.assertTrue("'widget.uservoice.com/abcdefghijklmnopqrst.js'" in r,
                r)

    def test_link_with_key_enables_tab(self):
        r = self.render_template('{% load uservoice %}'
                '{% uservoice_popup "efghijklmnopqrstuvwx" %}{% uservoice %}')
        self.assertTrue('UserVoice.showPopupWidget({"widget_key": '
                '"efghijklmnopqrstuvwx"});' in r, r)
        self.assertTrue('"enabled": true' in r, r)
        self.assertTrue("'widget.uservoice.com/abcdefghijklmnopqrst.js'" in r,
                r)

    def test_custom_fields(self):
        vars = {
            'uservoice_fields': {
                'field1': 'val1',
                'field2': 'val2',
            }
        }
        r = UserVoiceNode().render(Context(vars))
        self.assertTrue('"custom_fields": {"field2": "val2", "field1": "val1"}'
            in r, r)



"""
UserVoice template tags.
"""

from __future__ import absolute_import

import re

from django.template import Library, Node, TemplateSyntaxError, Variable
from django.utils import simplejson

from analytical.utils import get_identity, get_required_setting


WIDGET_KEY_RE = re.compile(r'^[a-zA-Z0-9]*$')
TRACKING_CODE = """
    <script type="text/javascript">
      var uvOptions = %(options)s;
      (function() {
        var uv = document.createElement('script'); uv.type = 'text/javascript'; uv.async = true;
        uv.src = ('https:' == document.location.protocol ? 'https://' : 'http://') + 'widget.uservoice.com/%(widget_key)s.js';
        var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(uv, s);
      })();
    </script>
"""
LINK_CODE = "UserVoice.showPopupWidget(%s);"


register = Library()


@register.tag
def uservoice(parser, token):
    """
    UserVoice tracking template tag.

    Renders Javascript code to track page visits.  You must supply
    your UserVoice Widget Key in the ``USERVOICE_WIDGET_KEY``
    setting or the ``uservoice_widget_key`` template context variable.
    """
    bits = token.split_contents()
    if len(bits) > 1:
        raise TemplateSyntaxError("'%s' takes no arguments" % bits[0])
    return UserVoiceNode()

class UserVoiceNode(Node):
    def __init__(self):
        self.default_widget_key = get_required_setting('USERVOICE_WIDGET_KEY',
                WIDGET_KEY_RE, "must be an alphanumeric string")

    def render(self, context):
        widget_key = context.get('uservoice_widget_key')
        if not widget_key:
            widget_key = self.default_widget_key
        if not widget_key:
            return ''
        options = {}
        options['enabled'] = context.get('uservoice_show_tab', True)
        options['custom_fields'] = context.get('uservoice_fields', {})
        identity = get_identity(context, 'uservoice')
        if identity is not None:
            # Enable SSO
            pass
        html = TRACKING_CODE % {'widget_key': widget_key,
                'options': simplejson.dumps(options)}
        return html


@register.tag
def uservoice_popup(parser, token):
    """
    UserVoice widget popup template tag.

    Renders the Javascript code to pop-up the UserVoice widget.  For example::

        <a href="#" onclick="{% uservoice_popup %}; return false;">Feedback</a>

    The tag accepts an optional argument specifying the key of the widget you
    want to show::

        <a href="#" onclick="{% uservoice_popup 'XXXXXXXXXXXXXXXXXXXX' %}; return false;">Helpdesk</a>

    If you add this tag without a widget key, the default feedback tab will be
    hidden.
    """
    bits = token.split_contents()
    if len(bits) == 1:
        return UserVoiceLinkNode()
    if len(bits) == 2:
        return UserVoiceKeyLinkNode(bits[1])
    raise TemplateSyntaxError("'%s' takes at most one argument" % bits[0])

class UserVoiceLinkNode(Node):
    def render(self, context):
        context['uservoice_show_tab'] = False
        return LINK_CODE % ''

class UserVoiceKeyLinkNode(Node):
    def __init__(self, widget_key):
        self.widget_key = Variable(widget_key)

    def render(self, context):
        vars = {}
        if self.widget_key:
            vars['widget_key'] = self.widget_key.resolve(context)
        return LINK_CODE % simplejson.dumps(vars)


def contribute_to_analytical(add_node):
    UserVoiceNode()  # ensure properly configured
    add_node('body_bottom', UserVoiceNode)
